import * as d3 from 'd3';
import { colorScale } from './utils';

export const bubbles = (selection, props) => {
  const { nodes, height, width } = props;
 
   const g = selection.selectAll('g')
  	.data([null])
    .enter()
  	.append('g');

  selection.call(d3.zoom().on('zoom', () => {
    g.attr('transform', d3.event.transform)
  }));


  function renderNodes(){
    var circles = g
      .selectAll('circle')
      .data(nodes)

    const circlesEnter = circles.enter()
      .append('circle')
    	.attr('class', 'node')
      .attr('r', d => d.depth === 0 ? 50 : 30)
      .call(d3.drag()
            .on("start", dragStarted)
            .on('drag', dragging)
            .on('end', dragEnded)
      )
    	.append('title')
    	.text(d => d.data.name);
    circlesEnter.merge(circles)
      .attr('fill', d => colorScale(d.depth === 0 ? 'ROOT' : d.data.type))
      .attr('cx', d => d.x)
      .attr('cy', d => d.y);

    circles.exit().remove();
  }

  var dragStarted = (d) => {
        if (!d3.event.active) simulation.alphaTarget(0.3).restart();
        d.fx = d.x;
        d.fy = d.y
      },
      dragging = (d) => {
        d.fx = d3.event.x;
        d.fy = d3.event.y
      },
      dragEnded = (d) => {
        if (!d3.event.active){
          simulation.alphaTarget(0);
          simulation.restart();
        }
        d.fx = null;
        d.fy = null
      }

  var simulation = d3.forceSimulation(nodes)
  	.velocityDecay(0.5)
  	.alpha(1)
  	.alphaDecay(1)
    .force('charge', d3.forceManyBody().strength(d => d.depth === 0 ? 20 : 10).distanceMin(0))
    .force('center', d3.forceCenter(width / 2, height / 2))
    .force('collision', d3.forceCollide().radius(function(d) {
      return d.depth === 0 ? 60 : 35
    }))
  	.alphaTarget(0.1)
    .on('tick', renderNodes);
}





