import * as d3 from 'd3';
import { colorScale } from './utils';

export const network = (selection, props) => {
  const { nodes, links, height, width } = props;
  
  const g = selection.selectAll('g')
  	.data([null])
    .enter()
  	.append('g');

  selection.call(d3.zoom().on('zoom', () => {
    g.attr('transform', d3.event.transform)
  }));

  function renderLines(){
    var lines = g
      .selectAll('line')
      .data(links)

    lines.enter()
      .append('line')
      .attr('stroke', 'black')
      .merge(lines)
      .attr('x1', function(d) {return d.source.x;})
      .attr('y1', function(d) {return d.source.y;})
      .attr('x2', function(d) {return d.target.x;})
      .attr('y2', function(d) {return d.target.y;});

    lines.exit().remove();
    
  }

  function renderNodes(){
    var circles = g
      .selectAll('circle')
      .data(nodes)

    var enterCircles = circles.enter()
      .append('circle')
      .attr('r', d => {
        if(d.depth === 0){
        	return 50
        } else if(!d.children){
          return 40
        }
        return 30
      })
    	.attr('class', 'node')
      .call(d3.drag()
            .on("start", dragStarted)
            .on('drag', dragging)
            .on('end', dragEnded)
      )
    	.append('title')
    	.text(d => d.data.name);
      
    enterCircles.merge(circles)
      .attr('fill', d => colorScale(d.depth === 0 ? 'ROOT' : d.data.type))
      .attr('cx', d => d.x)
      .attr('cy', d => d.y);

    circles.exit().remove();
  }

  function ticked() {
    renderLines();
    renderNodes();
  }

  var dragStarted = (d) => {
        if (!d3.event.active) simulation.alphaTarget(0.3).restart();
        d.fx = d.x;
        d.fy = d.y
      },
      dragging = (d) => {
        d.fx = d3.event.x;
        d.fy = d3.event.y
      },
      dragEnded = (d) => {
        if (!d3.event.active){
          simulation.alphaTarget(0);
          simulation.restart();
        }
        d.fx = null;
        d.fy = null
      }

  var simulation = d3.forceSimulation(nodes)
    .force('charge', d3.forceManyBody().strength(-50))
    .force('center', d3.forceCenter(width / 2, height / 2))
    .force('link', d3.forceLink().links(links))
    .force('collision', d3.forceCollide().radius((d) => {
      {
        if(d.depth === 0){
        	return 50
        } else if(!d.children){
          return 50
        }
        return 50
      }
    }))
    .on('tick', ticked);

}







